import os
import time
import sys

import sqlalchemy as sa
from sqlalchemy.orm import sessionmaker, scoped_session
import pandas as pd

sys.path.append('..')

import config


class DB(object):

    def __init__(self, source_db=None):
        if source_db is None:
            source_db = '''mysql+mysqlconnector://{0}:{1}@{2}:{3}/{4}'''.format(config.rl_db_user,
                                                                                config.rl_db_pwd,
                                                                                config.rl_db_host,
                                                                                config.rl_db_port,
                                                                                config.rl_db_database)
        # 源数据库
        self._source = sa.create_engine(source_db, pool_size=10)
        # 目标数据库Session
        self._session = sessionmaker(bind=self._source, autocommit=False, autoflush=True)
        self._scoped_session = scoped_session(self._session)
        self._clean_columns = set(['flag', 'is_verify', 'create_time', 'update_time'])

    def delete_data_by_id(self, ids, table):
        start_time = time.time()
        i, count = 0, 3000
        while i * count < len(ids):
            del_ids = ids[i * count:(i + 1) * count]
            sql = '''delete from `{0}` where id in {1}'''.format(table, del_ids)
            sql = sql.replace('[', '(').replace(']', ')')
            self._execute_sql(sql)
            i = i + 1
            print(f"delete id form {del_ids[0]} to {del_ids[-1]}")
        print('delete over.', time.time() - start_time)

    def save(self, df, table):
        df.to_sql(name=table, con=self._source, if_exists='append', index=False)

    def query_sql(self, sql, clean=False):
        start = time.perf_counter()
        result = pd.read_sql(sql, self._source)
        if clean:
            clean_columns = list(self._clean_columns & set(result.columns))
            result.drop(columns=clean_columns, inplace=True)
        print('query cost:', time.perf_counter() - start)
        return result

    def get_max_column(self, table=None, column='update_time'):
        sql = f'select max({column}) as tm from `{table}`'
        result = self.query_sql(sql)
        result['tm'] = result['tm'].astype('str')
        tm = None
        if not result.empty:
            tm = result['tm'][0]
        return tm

    def _execute_sql(self, sql):
        session = self._session()
        session.execute(sql)
        session.commit()
        session.close()

    def _insert_or_update(self, datas, table):
        session = self._session()
        for i in range(datas.shape[0]):
            data = datas.iloc[i]
            values = ''
            update = ''
            title = ''
            for j in range(len(data)):
                index = data.index[j]
                value = str(data[j]).replace("'", "\\'").replace("%", "\\%").replace("(", "\(").replace(")",
                                                                                                        "\)").replace(
                    ":", "\:")
                title += """`{0}`,""".format(index)
                values += """'{0}',""".format(value)
                update += """`{0}`='{1}',""".format(index, value)

            sql = '''insert into {0} ({1}) values({2}) ON DUPLICATE KEY UPDATE {3}'''.format(table,
                                                                                             title[0:-1],
                                                                                             values[0:-1],
                                                                                             update[0:-1]
                                                                                             )
            sql = sql.replace("'nan'", 'Null').replace("'None'", 'Null')
            try:
                session.execute(sql)
            except Exception as e:
                print(sql)
                print(e.orig.msg)
        session.commit()
        session.close()

    def _save_to_mysql(self, df, table):
        df.replace({',': '\,', '\n': '\\\\n', '\r\n': '\\\\n', '%': '\%'}, regex=True, inplace=True)
        tmpfile = table + '_mysql.csv'
        sep = ','
        newline = '\n'
        start = time.time()
        self._insert_or_update(df[:0])
        # df[:0].to_sql(name=self.dest_table, con=self.destination, if_exists='append', index=False)
        columns = df.columns.values.tolist()
        print('creating ' + tmpfile, 'ok')
        with open(tmpfile, mode='w', newline=newline) as fhandle:
            df.to_csv(fhandle, na_rep='\\N', index=False, sep=sep, encoding='UTF-8')
        # df.to_csv(tmpfile, na_rep='\\N', index=False, sep=sep, encoding='UTF-8')
        print('loading ' + tmpfile, 'ok')
        sql_load = "LOAD DATA LOCAL INFILE '{}' replace INTO TABLE {} FIELDS TERMINATED BY '{}' LINES TERMINATED BY '{}' IGNORE 1 LINES ({});" \
            .format(tmpfile, table, sep, newline, ','.join(columns))
        self._source.execute(sql_load)
        os.remove(tmpfile)
        end = time.time()
        print('存储数据库用时(s)：', end - start)
        return True
