#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
@version: ??
@author: zzh
@file: sqlengine.py
@time: 2019-08-29 16:22
"""

import datetime
import sys

sys.path.append('../..')

import config
import sqlalchemy as sa
import numpy as np
import pandas as pd
from sqlalchemy.orm import sessionmaker
from PyFin.api import advanceDateByCalendar


class InternalCode(object):
    def __init__(self):
        source_db = '''mysql+mysqlconnector://{0}:{1}@{2}:{3}/{4}'''.format(config.rl_db_user,
                                                                            config.rl_db_pwd,
                                                                            config.rl_db_host,
                                                                            config.rl_db_port,
                                                                            config.rl_db_database)
        # 源数据库
        self.source = sa.create_engine(source_db)
        # 数据库Session
        self.session = sessionmaker(bind=self.source, autocommit=False, autoflush=True)
        self.internal_code_table = 'gl_internal_code'
        self.sw_industry_table = 'sw_industry_daily'

    def get_sw_industry_daily(self, dates=None):
        trade_dates = {}
        for date in dates:
            trade_date = advanceDateByCalendar('china.sse', date, '-0b')
            trade_dates[trade_date] = date
        pass

    def get_Ashare_internal_code(self, trade_date=None, type=[], security_type=[101]):
        if trade_date == None:
            sql = """select security_code,symbol as code,exchange,company_id,security_name from `{0}`
                        where exchange in (001002,001003) and security_type in {1} and flag=1
                        and security_code!=2010004233""".format(
                self.internal_code_table, security_type).replace('[', '(').replace(']', ')')
            if 'security_code' in (type) or 'company_id' in (type):
                sql = sql + ' and is_valid=1; '
        else:
            sql = """select security_code,symbol as code,exchange,company_id,security_name,begin_date,end_date from `{0}`
                        where exchange in (001002,001003) and security_type in {1} and flag=1;""".format(
                self.internal_code_table, security_type).replace('[', '(').replace(']', ')')
        result_list = pd.read_sql(sql, self.source)
        if not result_list.empty:
            result_list['symbol'] = np.where(result_list['exchange'] == '001002',
                                             result_list['code'] + '.XSHG',
                                             result_list['code'] + '.XSHE')
            result_list.drop(['exchange'], axis=1, inplace=True)
        return result_list

    def get_Ashare_internal_code_list(self, trade_dates=[], security_type=[101]):
        if len(trade_dates) == 0:
            trade_dates = trade_dates.append(datetime.date.today())
        result_list = self.get_Ashare_internal_code(trade_dates, security_type=security_type)
        df = pd.DataFrame()
        if not result_list.empty:
            for trade_date in trade_dates:
                if not isinstance(trade_date, datetime.date):
                    trade_date = datetime.datetime.strptime(str(trade_date), '%Y%m%d').date()
                trade_date_code = result_list[
                    ((result_list['begin_date'] <= trade_date) & (result_list['end_date'] > trade_date)) | (
                            (result_list['begin_date'] <= trade_date) & (
                            result_list['end_date'] == datetime.datetime.strptime('19000101', '%Y%m%d').date()))]
                trade_date_code['trade_date'] = trade_date
                df = df.append(trade_date_code)
        df.drop(columns=['begin_date', 'end_date'], axis=1, inplace=True)
        df.reset_index(drop=True, inplace=True)
        return df

    def get_fd_internal_code(self, trade_date=None, type=[], security_type=[302]):
        if trade_date == None:
            sql = """select security_code,symbol as code,company_id,exchange,security_name from `{0}`
                        where exchange in (001002,001003,000091) and security_type in {1} and flag=1
                  """.format(self.internal_code_table, security_type).replace('[', '(').replace(']', ')')
        else:
            sql = """select security_code,symbol as code,exchange,company_id,security_name,begin_date,end_date from `{0}`
                        where exchange in (001002,001003,000091) and security_type in {1} and flag=1
                  """.format(self.internal_code_table, security_type).replace('[', '(').replace(']', ')')
        result_list = pd.read_sql(sql, self.source)
        if not result_list.empty:
            result_list['symbol'] = np.where(result_list['exchange'] == '001002',
                                             result_list['code'] + '.XSHG',
                                             result_list['code'])
            result_list['symbol'] = np.where(result_list['exchange'] == '001003',
                                             result_list['code'] + '.XSHE',
                                             result_list['symbol'])
            result_list['symbol'] = np.where(result_list['exchange'] == '000091',
                                             result_list['code'] + '.OFCN',
                                             result_list['symbol'])
            result_list.drop(['exchange'], axis=1, inplace=True)
        return result_list

    def get_fd_internal_code_list(self, trade_dates=[], security_type=[302]):
        if len(trade_dates) == 0:
            trade_dates = trade_dates.append(datetime.date.today())
        result_list = self.get_Ashare_internal_code(trade_dates, security_type=security_type)
        df = pd.DataFrame()
        if not result_list.empty:
            for trade_date in trade_dates:
                if not isinstance(trade_date, datetime.date):
                    trade_date = datetime.datetime.strptime(str(trade_date), '%Y%m%d').date()
                trade_date_code = result_list[
                    ((result_list['begin_date'] <= trade_date) & (result_list['end_date'] > trade_date)) | (
                            (result_list['begin_date'] <= trade_date) & (
                            result_list['end_date'] == datetime.datetime.strptime('19000101', '%Y%m%d').date()))]
                trade_date_code['trade_date'] = trade_date
                df = df.append(trade_date_code)
        df.drop(columns=['begin_date', 'end_date'], axis=1, inplace=True)
        df.reset_index(drop=True, inplace=True)
        return df

    def join_internal_code(self, df, left=[], right=[], include_name=False, security_type=[101], data_type='stk',
                           style='basic'):
        if data_type == 'stk':
            get_internal_code_list = self.get_Ashare_internal_code_list
            get_internal_code = self.get_Ashare_internal_code
        else:
            get_internal_code_list = self.get_fd_internal_code_list
            get_internal_code = self.get_fd_internal_code
            security_type = [302]
        if not df.empty:
            if ('trade_date' in left):
                trade_dates = list(set(df['trade_date']))
                trade_dates.sort()
                internal_code_sets = get_internal_code_list(trade_dates, security_type=security_type)
            else:
                internal_code_sets = get_internal_code(type=left, security_type=security_type)
            if not include_name:
                internal_code_sets.drop(columns=['security_name'], inplace=True)
            internal_code_sets.drop(columns=['company_id'], inplace=True)
            df = pd.merge(df, internal_code_sets, left_on=left, right_on=right)
            if style == 'basic':
                df = df.drop(columns=['symbol']).rename(columns={'code': 'symbol'})
        return df

    def transfer_internal_code(self, column, value, security_type=101, trade_date=None):
        date_str = ""
        if trade_date is not None:
            date_str = f" and ((begin_date<='{trade_date}' and end_date>'{trade_date}') " \
                       f"or (begin_date<='{trade_date}' and end_date='19000101')"
        sql = f"select company_id,security_code,symbol,exchange from {self.internal_code_table} " \
              f"where exchange in (001002,001003) and security_type={security_type} and is_valid=1 and flag=1 " \
              f"and {column}={value} {date_str}; "
        result = pd.read_sql(sql, self.source)
        if result.empty:
            return None
        else:
            result['symbol'] = np.where(result['exchange'] == '001002',
                                        result['symbol'] + '.XSHG',
                                        result['symbol'] + '.XSHE')
            return {
                'company_id': result['company_id'][0],
                'security_code': result['security_code'][0],
                'symbol': result['symbol'][0]
            }


if __name__ == '__main__':
    internal = InternalCode()
    # sets = internal.get_Ashare_internal_code('20190108')
    # print(sets)
    # sets = internal.get_Ashare_internal_code_list(['2010-07-31'])
    # sets = internal.get_Ashare_internal_code_list(['20190108'])
    combine_df = pd.read_csv('../files/tf-6.csv', encoding='utf-8', sep=",")
    combine_df['trade_date'] = pd.to_datetime(combine_df['times']).dt.date
    combine_df['code'] = combine_df['codes'].apply(lambda x: x[0:6])
    combine_df_t = internal.join_internal_code(combine_df, left=['trade_date', 'code'],
                                               right=['trade_date', 'code'])
    print(combine_df_t)
